##======================================================================
## Examples 
##======================================================================

N <- rpois(3,100)
id <- factor(rep(c("A","B","C"),times=N))
positions <- unlist(tapply(rexp(sum(N)),id,cumsum))
positions <- c(positions,positions + runif(sum(N),0,1))
id <- rep(id,2)
N <- 2*N


tmpMPP <- new("MarkedPointProcess",
              id=id,
              position=positions,
              markType=factor(sample(c("ALPHA","BETA"),sum(N),replace=TRUE)),
              markValue=data.frame(length=rep(2,sum(N))))

ll <- tapply(positions,id,max)
position <- sapply(ll,function(l) seq(0,l+0.1,by=0.1))
id <- factor(rep(c("A","B","C"),times=sapply(position,length)))
position <- unlist(position)

value <- data.frame(val1=rnorm(length(position),rnorm(10)[id] + log(position+1)),
                    val2=rnorm(length(position),rnorm(10)[id] - log(position+1)))
initdata <- data.frame(id=id,position=position,value=value)
tmpCP <- continuousProcess(initdata[,1:4])
tmpPPD <- new(Class="ProcessData",
              markedPointProcess=tmpMPP,
              continuousProcess=tmpCP)

### tmpCP
### plot(tmpCP)


## plot(tmpMPP)
## tmpCP <- mpp2cp(tmpMPP,1:300/10)
## tmpCP <- mpp2cp(tmpMPP,1:300/10,process="pointProcess")
## tmpCP
## plot(tmpCP)

#getValue(tmpCP,50)


plot(tmpPPD)+facet_grid(processType~id,scale="free_y",space="free")

plot(subset(tmpPPD,id %in% c("A","B"))) + facet_grid(processType~id,scale="free_y",space="free")

plot(subset(tmpMPP,id=="A"))

plot(subset(tmpCP,id=="A"))

### Creating a PointProcessModel object by hand

## tmpPPM <- new(Class="PointProcessModel",
##               processData=tmpPPD,
##               family = new("Family",name="Hawkes",link="log"),
##               ## formula=as.formula("K~id"),
##               formula=as.formula("K~ bSpline(K,knots=seq(-0.1,8,0.1),whichColumns=-c(1,81))+tExp(-K %*% t(seq(1,5,0.1)))"),
##               coefficients = c(1,rep(0,118)),
##               modelMatrix=FALSE)

## i <- tmpPPD@pointPosition$where[1]
## tmp <- subset(tmpPPD, position %in% getContinuousProcess(tmpPPD)@position[c(i,i+1)])

## test <- new(Class="PointProcessModel",
##               processData=tmp,
##               family = new("Family",name="Hawkes",link="log"),
##               formula=ALPHA~ bSpline(ALPHA,knots=seq(0,5,0.5))+bSpline(BETA,knots=seq(0,5,0.5)),
##               modelMatrix=TRUE)

### creating it by a call to glppm that does the estimation also.

#tmpPPM <- glppm(K~ bSpline(APLHA,knots=seq(-0.1,8,1))+tExp(-ALPHA),data=tmpPPD,family=new("Family",name="Hawkes",link="log"))
tmpPPM <- glppm(ALPHA~ id*value.val1+bSpline(ALPHA,knots=seq(0,2,0.4))+bSpline(BETA,knots=seq(0,2,0.4)),data=tmpPPD,family=new("Family",name="Hawkes",link="log"))

plot(seq(0,3,0.01),tmpPPM)
## updatedPPM <- update(tmpPPM,.~.-bSpline(ALPHA,knots=seq(0,2,0.4))) # This does reuse the model matrix correctly.
## updatedPPM <- update(updatedPPM,.~.)


tmp2PPM <- glppm(ALPHA~ bSpline(ALPHA,knots=seq(0,2,0.1))+bSpline(BETA,knots=seq(0,2,0.1)),data=tmpPPD,family=new("Family",name="Hawkes",link="log"))
tmp3PPM <- glppm(ALPHA~ id-1,data=tmpPPD,family=new("Family",name="Hawkes",link="log"))
summary(tmp3PPM)


### Trying to fix one of the coefficients
tmpPPM <- glppm(ALPHA~ id*value.val1+bSpline(ALPHA,knots=seq(0,2,0.4))+bSpline(BETA,knots=seq(0,2,0.4)),data=tmpPPD,family=new("Family",name="Hawkes",link="log"), fixedPar = list(which=c(3,4),value=c(-30,0.01523)))
summary(tmpPPM)


### Trying to fix one of the coefficients
tmpPPM <- glppm(ALPHA~ id*value.val1+bSpline(ALPHA,knots=seq(0,2,0.4))+bSpline(BETA,knots=seq(0,2,0.4)),data=tmpPPD,family=new("Family",name="Hawkes",link="log"),Omega=diag(c(rep(0.01,6),rep(0,4))))
summary(tmpPPM)


res <- computeDDMinusLogLikelihood(tmpPPM)


### Investigating the relation between the 

logintensity <- getContinuousProcess(tmpPPD)
logintensity@value <- Matrix(computeLinearPredictor(tmpPPM))

PPD <- new(Class="ProcessData",
              markedPointProcess=tmpMPP,
              continuousProcess=logintensity)

plot(subset(PPD,id=="A"))



